import java.net.*;
import java.io.*;
import java.util.*;

public class BroadcastThreadService implements Runnable {

	private Socket clientSocket;
	private static ArrayList<PrintWriter> listWriter;

	BroadcastThreadService(Socket clientSocket) {
		System.out.println("Creation d'un thread pour repondre a un client, port "
		 					+ clientSocket.getPort());
		this.clientSocket = clientSocket;
		if (listWriter == null)
			listWriter = new ArrayList<PrintWriter>();
	}

	public void run() {
		try {
			doService(clientSocket, listWriter);
			clientSocket.close();
		}
		catch (IOException e) {
			System.err.println("IOException : " + e);
			e.printStackTrace();
		}
		finally {
			try {
				if (this.clientSocket != null)
					this.clientSocket.close();
			}
			catch (IOException e) {
				System.err.println("IOException : " + e);
				e.printStackTrace();
			}
		}
	}

	/**
	 * Be careful, this function must be thread-safe
	 * @param  clientSocket [description]
	 * @param  sharedList   [description]
	 * @throws IOException  [description]
	 */
	public void doService(Socket clientSocket, ArrayList<PrintWriter> sharedList) throws IOException {
		PrintWriter OWriter = new PrintWriter(clientSocket.getOutputStream());
		BufferedReader in;
		in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
		sharedList.add(OWriter);
		boolean end = false;
		while (!end) {
			String theLine = in.readLine();
			if (theLine.equals(".")) {
				end = true; // le thread de service doit terminer
				break; // do not broadcast the dot that will close all clients threads
			}
			System.out.println("Broadcasting the message \"" + theLine + "\" received from " + clientSocket.toString());
			broadcastMsg(theLine);
		}
		sharedList.remove(OWriter);
		if (OWriter != null)
			OWriter.close();
		if (in != null)
			in.close();
		System.out.println("Fin du thread repondant au client, port "
		 					+ clientSocket.getPort());
	}

	private void broadcastMsg(String msg) {
		ListIterator<PrintWriter> iter = listWriter.listIterator();
		while (iter.hasNext()) {
			PrintWriter cursorPW = iter.next();
			cursorPW.println(msg);
			cursorPW.flush();
		}
	}

}
